@@ -38,9 +38,15 @@ non-Python code run under Apache.
 """
 
 import time as time_mod
-import md5
+try:
+    from hashlib import md5
+except ImportError:
+    from md5 import md5
 import Cookie
 from paste import request
+from urllib import quote as url_quote
+from urllib import unquote as url_unquote
+
 
 class AuthTicket(object):
 
@@ -99,7 +105,7 @@ class AuthTicket(object):
             self.user_data)
 
     def cookie_value(self):
-        v = '%s%08x%s!' % (self.digest(), int(self.time), self.userid)
+        v = '%s%08x%s!' % (self.digest(), int(self.time), url_quote(self.userid))
         if self.tokens:
             v += self.tokens + '!'
         v += self.user_data
@@ -113,6 +119,7 @@ class AuthTicket(object):
             c[self.cookie_name]['secure'] = 'true'
         return c
 
+
 class BadTicket(Exception):
     """
     Exception raised when a ticket can't be parsed.  If we get
@@ -124,6 +131,7 @@ class BadTicket(Exception):
         self.expected = expected
         Exception.__init__(self, msg)
 
+
 def parse_ticket(secret, ticket, ip):
     """
     Parse the ticket, returning (timestamp, userid, tokens, user_data).
@@ -141,13 +149,14 @@ def parse_ticket(secret, ticket, ip):
         userid, data = ticket[40:].split('!', 1)
     except ValueError:
         raise BadTicket('userid is not followed by !')
+    userid = url_unquote(userid)
     if '!' in data:
         tokens, user_data = data.split('!', 1)
     else:
         # @@: Is this the right order?
         tokens = ''
         user_data = data
-    
+
     expected = calculate_digest(ip, timestamp, secret,
                                 userid, tokens, user_data)
 
@@ -158,14 +167,20 @@ def parse_ticket(secret, ticket, ip):
     tokens = tokens.split(',')
 
     return (timestamp, userid, tokens, user_data)
-    
+
+
 def calculate_digest(ip, timestamp, secret, userid, tokens, user_data):
-    digest0 = md5.new(
+    secret = maybe_encode(secret)
+    userid = maybe_encode(userid)
+    tokens = maybe_encode(tokens)
+    user_data = maybe_encode(user_data)
+    digest0 = md5(
         encode_ip_timestamp(ip, timestamp) + secret + userid + '\0'
         + tokens + '\0' + user_data).hexdigest()
-    digest = md5.new(digest0 + secret).hexdigest()
+    digest = md5(digest0 + secret).hexdigest()
     return digest
 
+
 def encode_ip_timestamp(ip, timestamp):
     ip_chars = ''.join(map(chr, map(int, ip.split('.'))))
     t = int(timestamp)
@@ -177,6 +192,12 @@ def encode_ip_timestamp(ip, timestamp):
     return ip_chars + ts_chars
 
 
+def maybe_encode(s, encoding='utf8'):
+    if isinstance(s, unicode):
+        s = s.encode(encoding)
+    return s
+
+
 class AuthTKTMiddleware(object):
 
     """
@@ -187,29 +208,33 @@ class AuthTKTMiddleware(object):
     you).
 
     Arguments:
-    
+
     ``secret``:
         A secret that should be shared by any instances of this application.
-        If this app is served from more than one machine, they should all 
+        If this app is served from more than one machine, they should all
         have the same secret.
-        
+
     ``cookie_name``:
         The name of the cookie to read and write from.  Default ``auth_tkt``.
-        
+
     ``secure``:
         If the cookie should be set as 'secure' (only sent over SSL) and if
-        the login must be over SSL.
-        
+        the login must be over SSL. (Defaults to False)
+
+    ``httponly``:
+        If the cookie should be marked as HttpOnly, which means that it's
+        not accessible to JavaScript. (Defaults to False)
+
     ``include_ip``:
-        If the cookie should include the user's IP address.  If so, then 
+        If the cookie should include the user's IP address.  If so, then
         if they change IPs their cookie will be invalid.
-        
+
     ``logout_path``:
         The path under this middleware that should signify a logout.  The
         page will be shown as usual, but the user will also be logged out
         when they visit this page.
-        
-    If used with mod_auth_tkt, then these settings (except logout_path) should 
+
+    If used with mod_auth_tkt, then these settings (except logout_path) should
     match the analogous Apache configuration settings.
 
     This also adds two functions to the request:
@@ -226,17 +251,23 @@ class AuthTKTMiddleware(object):
     """
 
     def __init__(self, app, secret, cookie_name='auth_tkt', secure=False,
-                 include_ip=True, logout_path=None):
+                 include_ip=True, logout_path=None, httponly=False,
+                 no_domain_cookie=True, current_domain_cookie=True,
+                 wildcard_cookie=True):
         self.app = app
         self.secret = secret
         self.cookie_name = cookie_name
         self.secure = secure
+        self.httponly = httponly
         self.include_ip = include_ip
         self.logout_path = logout_path
+        self.no_domain_cookie = no_domain_cookie
+        self.current_domain_cookie = current_domain_cookie
+        self.wildcard_cookie = wildcard_cookie
 
     def __call__(self, environ, start_response):
         cookies = request.get_cookies(environ)
-        if cookies.has_key(self.cookie_name):
+        if self.cookie_name in cookies:
             cookie_value = cookies[self.cookie_name].value
         else:
             cookie_value = ''
@@ -249,29 +280,39 @@ class AuthTKTMiddleware(object):
                 remote_addr = '0.0.0.0'
             # @@: This should handle bad signatures better:
             # Also, timeouts should cause cookie refresh
-            timestamp, userid, tokens, user_data = parse_ticket(
-                self.secret, cookie_value, remote_addr)
-            tokens = ','.join(tokens)
-            environ['REMOTE_USER'] = userid
-            if environ.get('REMOTE_USER_TOKENS'):
-                # We want to add tokens/roles to what's there:
-                tokens = environ['REMOTE_USER_TOKENS'] + ',' + tokens
-            environ['REMOTE_USER_TOKENS'] = tokens
-            environ['REMOTE_USER_DATA'] = user_data
-            environ['AUTH_TYPE'] = 'cookie'
+            try:
+                timestamp, userid, tokens, user_data = parse_ticket(
+                    self.secret, cookie_value, remote_addr)
+                tokens = ','.join(tokens)
+                environ['REMOTE_USER'] = userid
+                if environ.get('REMOTE_USER_TOKENS'):
+                    # We want to add tokens/roles to what's there:
+                    tokens = environ['REMOTE_USER_TOKENS'] + ',' + tokens
+                environ['REMOTE_USER_TOKENS'] = tokens
+                environ['REMOTE_USER_DATA'] = user_data
+                environ['AUTH_TYPE'] = 'cookie'
+            except BadTicket:
+                # bad credentials, just ignore without logging the user
+                # in or anything
+                pass
         set_cookies = []
+
         def set_user(userid, tokens='', user_data=''):
             set_cookies.extend(self.set_user_cookie(
                 environ, userid, tokens, user_data))
+
         def logout_user():
             set_cookies.extend(self.logout_user_cookie(environ))
+
         environ['paste.auth_tkt.set_user'] = set_user
         environ['paste.auth_tkt.logout_user'] = logout_user
         if self.logout_path and environ.get('PATH_INFO') == self.logout_path:
             logout_user()
+
         def cookie_setting_start_response(status, headers, exc_info=None):
             headers.extend(set_cookies)
             return start_response(status, headers, exc_info)
+
         return self.app(environ, cookie_setting_start_response)
 
     def set_user_cookie(self, environ, userid, tokens, user_data):
@@ -291,23 +332,44 @@ class AuthTKTMiddleware(object):
             secure=self.secure)
         # @@: Should we set REMOTE_USER etc in the current
         # environment right now as well?
-        cookies = [
-            ('Set-Cookie', '%s=%s; Path=/' % (
-            self.cookie_name, ticket.cookie_value()))]
+        cur_domain = environ.get('HTTP_HOST', environ.get('SERVER_NAME'))
+        wild_domain = '.' + cur_domain
+
+        cookie_options = ""
+        if self.secure:
+            cookie_options += "; secure"
+        if self.httponly:
+            cookie_options += "; HttpOnly"
+
+        cookies = []
+        if self.no_domain_cookie:
+            cookies.append(('Set-Cookie', '%s=%s; Path=/%s' % (
+                self.cookie_name, ticket.cookie_value(), cookie_options)))
+        if self.current_domain_cookie:
+            cookies.append(('Set-Cookie', '%s=%s; Path=/; Domain=%s%s' % (
+                self.cookie_name, ticket.cookie_value(), cur_domain,
+                cookie_options)))
+        if self.wildcard_cookie:
+            cookies.append(('Set-Cookie', '%s=%s; Path=/; Domain=%s%s' % (
+                self.cookie_name, ticket.cookie_value(), wild_domain,
+                cookie_options)))
+
         return cookies
-    
+
     def logout_user_cookie(self, environ):
         cur_domain = environ.get('HTTP_HOST', environ.get('SERVER_NAME'))
         wild_domain = '.' + cur_domain
+        expires = 'Sat, 01-Jan-2000 12:00:00 GMT'
         cookies = [
-            ('Set-Cookie', '%s=""; Path=/' % self.cookie_name),
-            ('Set-Cookie', '%s=""; Path=/; Domain=%s' %
-             (self.cookie_name, cur_domain)),
-            ('Set-Cookie', '%s=""; Path=/; Domain=%s' %
-             (self.cookie_name, wild_domain)),
+            ('Set-Cookie', '%s=""; Expires="%s"; Path=/' % (self.cookie_name, expires)),
+            ('Set-Cookie', '%s=""; Expires="%s"; Path=/; Domain=%s' %
+             (self.cookie_name, expires, cur_domain)),
+            ('Set-Cookie', '%s=""; Expires="%s"; Path=/; Domain=%s' %
+             (self.cookie_name, expires, wild_domain)),
             ]
         return cookies
 
+
 def make_auth_tkt_middleware(
     app,
     global_conf,
