@@ -41,7 +41,7 @@ a WSGI collection, for removing and setting header values.
 
     This method removes all entries of the corresponding header from
     the given collection (``environ`` or ``response_headers``), e.g.,
-    ``USER_AGENT.remove(environ)`` deletes the 'HTTP_USER_AGENT' entry
+    ``USER_AGENT.delete(environ)`` deletes the 'HTTP_USER_AGENT' entry
     from the ``environ``.
 
   ``update(collection, *args, **kwargs)``
@@ -134,9 +134,9 @@ dashes to give CamelCase style names.
 .. [3] http://www.python.org/peps/pep-0333.html#the-start-response-callable
 
 """
+import mimetypes
 import urllib2
 import re
-from mimetypes import guess_type
 from rfc822 import formatdate, parsedate_tz, mktime_tz
 from time import time as now
 from httpexceptions import HTTPBadRequest
@@ -459,7 +459,7 @@ class HTTPHeader(object):
         """
         value = self.__call__(*args, **kwargs)
         if not value:
-            self.remove(collection)
+            self.delete(collection)
             return
         if type(collection) == dict:
             collection[self._environ_name] = value
@@ -575,7 +575,7 @@ def normalize_headers(response_headers, strict=True):
         (key, val) = response_headers[idx]
         head = get_header(key, strict)
         if not head:
-            newhead = '-'.join([x.capitalize() for x in 
+            newhead = '-'.join([x.capitalize() for x in
                                 key.replace("_","-").split("-")])
             response_headers[idx] = (newhead, val)
             category[newhead] = 4
@@ -617,7 +617,7 @@ class _DateHeader(_SingleValueHeader):
         if value:
             try:
                 return mktime_tz(parsedate_tz(value))
-            except TypeError:
+            except (TypeError, OverflowError):
                 raise HTTPBadRequest((
                     "Received an ill-formed timestamp for %s: %s\r\n") %
                     (self.name, value))
@@ -855,7 +855,7 @@ class _ContentDisposition(_SingleValueHeader):
         (result, filename) = self._compose(**kwargs)
         mimetype = CONTENT_TYPE(collection)
         if filename and (not mimetype or CONTENT_TYPE.UNKNOWN == mimetype):
-            mimetype, _ = guess_type(filename)
+            mimetype, _ = mimetypes.guess_type(filename)
             if mimetype and CONTENT_TYPE.UNKNOWN != mimetype:
                 CONTENT_TYPE.update(collection, mimetype)
         self.update(collection, *result)
@@ -913,7 +913,7 @@ class _Range(_MultiValueHeader):
         ranges = []
         last_end   = -1
         try:
-            (units, range) = value.split("=")
+            (units, range) = value.split("=", 1)
             units = units.strip().lower()
             for item in range.split(","):
                 (begin, end) = item.split("-")
@@ -986,7 +986,7 @@ class _ContentRange(_SingleValueHeader):
     Content-Range, RFC 2616 section 14.6
     """
     def compose(self, first_byte=None, last_byte=None, total_length=None):
-        retval = "%d-%d/%d" % (first_byte, last_byte, total_length)
+        retval = "bytes %d-%d/%d" % (first_byte, last_byte, total_length)
         assert last_byte == -1 or first_byte <= last_byte
         assert last_byte  < total_length
         return (retval,)
