@@ -72,7 +72,7 @@ class ContinueHook(object):
 
     def _ContinueFile_read(self, size=-1):
         self._ContinueFile_send()
-        return self._ContinueFile_rfile.readline(size)
+        return self._ContinueFile_rfile.read(size)
 
     def _ContinueFile_readline(self, size=-1):
         self._ContinueFile_send()
@@ -183,7 +183,7 @@ class WSGIHandlerMixin:
         if endslash and path != '/':
             # Put the slash back...
             path += '/'
-        (server_name, server_port) = self.server.server_address
+        (server_name, server_port) = self.server.server_address[:2]
 
         rfile = self.rfile
         if 'HTTP/1.1' == self.protocol_version and \
@@ -241,19 +241,19 @@ class WSGIHandlerMixin:
                 address_string = None # self.address_string()
                 if address_string:
                     self.wsgi_environ['REMOTE_HOST'] = address_string
-                
+
         if hasattr(self.server, 'thread_pool'):
             # Now that we know what the request was for, we should
             # tell the thread pool what its worker is working on
             self.server.thread_pool.worker_tracker[thread.get_ident()][1] = self.wsgi_environ
             self.wsgi_environ['paste.httpserver.thread_pool'] = self.server.thread_pool
-        
+
         for k, v in self.headers.items():
             key = 'HTTP_' + k.replace("-","_").upper()
             if key in ('HTTP_CONTENT_TYPE','HTTP_CONTENT_LENGTH'):
                 continue
             self.wsgi_environ[key] = ','.join(self.headers.getheaders(k))
-        
+
         if hasattr(self.connection,'get_context'):
             self.wsgi_environ['wsgi.url_scheme'] = 'https'
             # @@: extract other SSL parameters from pyOpenSSL at...
@@ -323,9 +323,11 @@ except ImportError:
     SocketErrors = (socket.error,)
     class SecureHTTPServer(HTTPServer):
         def __init__(self, server_address, RequestHandlerClass,
-                     ssl_context=None):
+                     ssl_context=None, request_queue_size=None):
             assert not ssl_context, "pyOpenSSL not installed"
             HTTPServer.__init__(self, server_address, RequestHandlerClass)
+            if request_queue_size:
+                self.socket.listen(request_queue_size)
 else:
 
     class _ConnFixer(object):
@@ -351,7 +353,7 @@ else:
         """
 
         def __init__(self, server_address, RequestHandlerClass,
-                     ssl_context=None):
+                     ssl_context=None, request_queue_size=None):
             # This overrides the implementation of __init__ in python's
             # SocketServer.TCPServer (which BaseHTTPServer.HTTPServer
             # does not override, thankfully).
@@ -367,8 +369,16 @@ else:
                             return self._ssl_conn.settimeout(*args)
                         finally:
                             self._lock.release()
+                    def gettimeout(self):
+                        self._lock.acquire()
+                        try:
+                            return self._ssl_conn.gettimeout()
+                        finally:
+                            self._lock.release()
                 self.socket = TSafeConnection(ssl_context, self.socket)
             self.server_bind()
+            if request_queue_size:
+                self.socket.listen(request_queue_size)
             self.server_activate()
 
         def get_request(self):
@@ -435,7 +445,7 @@ class WSGIHandler(WSGIHandlerMixin, BaseHTTPRequestHandler):
 
     def address_string(self):
         """Return the client address formatted for logging.
-        
+
         This is overridden so that no hostname lookup is done.
         """
         return ''
@@ -445,6 +455,8 @@ class LimitedLengthFile(object):
         self.file = file
         self.length = length
         self._consumed = 0
+        if hasattr(self.file, 'seek'):
+            self.seek = self._seek
 
     def __repr__(self):
         base_repr = repr(self.file)
@@ -464,7 +476,10 @@ class LimitedLengthFile(object):
         return data
 
     def readline(self, *args):
-        data = self.file.readline(self.length - self._consumed)
+        max_read = self.length - self._consumed
+        if len(args):
+            max_read = min(args[0], max_read)
+        data = self.file.readline(max_read)
         self._consumed += len(data)
         return data
 
@@ -484,7 +499,7 @@ class LimitedLengthFile(object):
 
     ## Optional methods ##
 
-    def seek(self, place):
+    def _seek(self, place):
         self.file.seek(place)
         self._consumed = place
 
@@ -540,7 +555,7 @@ class ThreadPool(object):
     dies and replaces itself with a new worker thread.
     """
 
-    
+
     SHUTDOWN = object()
 
     def __init__(
@@ -1007,7 +1022,7 @@ class ThreadPool(object):
         server.sendmail(from_address, error_emails, message)
         server.quit()
         print 'email sent to', error_emails, message
-        
+
 class ThreadPoolMixIn(object):
     """
     Mix-in class to process requests from a thread pool
@@ -1055,6 +1070,9 @@ class ThreadPoolMixIn(object):
         except:
             self.handle_error(request, client_address)
             self.close_request(request)
+            exc = sys.exc_info()[1]
+            if isinstance(exc, (MemoryError, KeyboardInterrupt)):
+                raise
 
     def serve_forever(self):
         """
@@ -1077,7 +1095,6 @@ class ThreadPoolMixIn(object):
         """
         # We set the timeout here so that we can trap interrupts on windows
         self.socket.settimeout(1)
-        self.socket.listen(self.request_queue_size)
 
     def server_close(self):
         """
@@ -1089,9 +1106,11 @@ class ThreadPoolMixIn(object):
 
 class WSGIServerBase(SecureHTTPServer):
     def __init__(self, wsgi_application, server_address,
-                 RequestHandlerClass=None, ssl_context=None):
+                 RequestHandlerClass=None, ssl_context=None,
+                 request_queue_size=None):
         SecureHTTPServer.__init__(self, server_address,
-                                  RequestHandlerClass, ssl_context)
+                                  RequestHandlerClass, ssl_context,
+                                  request_queue_size=request_queue_size)
         self.wsgi_application = wsgi_application
         self.wsgi_socket_timeout = None
 
@@ -1109,9 +1128,10 @@ class WSGIThreadPoolServer(ThreadPoolMixIn, WSGIServerBase):
     def __init__(self, wsgi_application, server_address,
                  RequestHandlerClass=None, ssl_context=None,
                  nworkers=10, daemon_threads=False,
-                 threadpool_options=None):
+                 threadpool_options=None, request_queue_size=None):
         WSGIServerBase.__init__(self, wsgi_application, server_address,
-                                RequestHandlerClass, ssl_context)
+                                RequestHandlerClass, ssl_context,
+                                request_queue_size=request_queue_size)
         if threadpool_options is None:
             threadpool_options = {}
         ThreadPoolMixIn.__init__(self, nworkers, daemon_threads,
@@ -1127,7 +1147,7 @@ def serve(application, host=None, port=None, handler=None, ssl_pem=None,
           ssl_context=None, server_version=None, protocol_version=None,
           start_loop=True, daemon_threads=None, socket_timeout=None,
           use_threadpool=None, threadpool_workers=10,
-          threadpool_options=None):
+          threadpool_options=None, request_queue_size=5):
     """
     Serves your ``application`` over HTTP(S) via WSGI interface
 
@@ -1223,6 +1243,12 @@ def serve(application, host=None, port=None, handler=None, ssl_pem=None,
         threadpool.  See paste.httpserver.ThreadPool for specific
         options (``threadpool_workers`` is a specific option that can
         also go here).
+
+    ``request_queue_size``
+
+        The 'backlog' argument to socket.listen(); specifies the
+        maximum number of queued connections.
+
     """
     is_ssl = False
     if ssl_pem or ssl_context:
@@ -1238,7 +1264,7 @@ def serve(application, host=None, port=None, handler=None, ssl_pem=None,
                 ssl_context.use_certificate_chain_file(ssl_pem)
 
     host = host or '127.0.0.1'
-    if not port:
+    if port is None:
         if ':' in host:
             host, port = host.split(':', 1)
         else:
@@ -1261,9 +1287,11 @@ def serve(application, host=None, port=None, handler=None, ssl_pem=None,
         server = WSGIThreadPoolServer(application, server_address, handler,
                                       ssl_context, int(threadpool_workers),
                                       daemon_threads,
-                                      threadpool_options=threadpool_options)
+                                      threadpool_options=threadpool_options,
+                                      request_queue_size=request_queue_size)
     else:
-        server = WSGIServer(application, server_address, handler, ssl_context)
+        server = WSGIServer(application, server_address, handler, ssl_context,
+                            request_queue_size=request_queue_size)
         if daemon_threads:
             server.daemon_threads = daemon_threads
 
@@ -1272,7 +1300,7 @@ def serve(application, host=None, port=None, handler=None, ssl_pem=None,
 
     if converters.asbool(start_loop):
         protocol = is_ssl and 'https' or 'http'
-        host, port = server.server_address
+        host, port = server.server_address[:2]
         if host == '0.0.0.0':
             print 'serving on 0.0.0.0:%s view at %s://127.0.0.1:%s' % \
                 (port, protocol, port)
@@ -1296,7 +1324,7 @@ def server_runner(wsgi_app, global_conf, **kwargs):
                  'threadpool_dying_limit', 'threadpool_spawn_if_under',
                  'threadpool_max_zombie_threads_before_die',
                  'threadpool_hung_check_period',
-                 'threadpool_max_requests']:
+                 'threadpool_max_requests', 'request_queue_size']:
         if name in kwargs:
             kwargs[name] = int(kwargs[name])
     for name in ['use_threadpool', 'daemon_threads']:
@@ -1313,7 +1341,7 @@ def server_runner(wsgi_app, global_conf, **kwargs):
     kwargs['threadpool_options'] = threadpool_options
     serve(wsgi_app, **kwargs)
 
-server_runner.__doc__ = serve.__doc__ + """
+server_runner.__doc__ = (serve.__doc__ or '') + """
 
     You can also set these threadpool options:
 
@@ -1372,6 +1400,7 @@ server_runner.__doc__ = serve.__doc__ + """
 
         When threads are killed or the process restarted, this email
         address will be contacted (using an SMTP server on localhost).
+
 """
 
 
@@ -1380,4 +1409,3 @@ if __name__ == '__main__':
     #serve(dump_environ, ssl_pem="test.pem")
     serve(dump_environ, server_version="Wombles/1.0",
           protocol_version="HTTP/1.1", port="8888")
-
