@@ -4,7 +4,6 @@
 from paste.fixture import *
 from paste.request import *
 from paste.wsgiwrappers import WSGIRequest
-from py.test import raises
 
 def simpleapp(environ, start_response):
     status = '200 OK'
@@ -59,3 +58,5 @@ def test_bad_cookie():
     assert get_cookie_dict(env) == {}
     env['HTTP_COOKIE'] = '=foo'
     assert get_cookie_dict(env) == {}
+    env['HTTP_COOKIE'] = '?='
+    assert get_cookie_dict(env) == {}
